#!/bin/sh
#  rrd-graph.cgi  a script to generate graphs with stats from RRD 
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
# Author: nemo@invisiblehand.net
# Version: $Revision: 1.18 $, $Date: 2003/07/12 06:15:57 $
# Copyright (C) 2002-2003 by Invisible Hand Networks, Inc.
#
# Usage: 
#    sh rrd-graph.cgi [<flow-name> [<day|week|month|year>]]
#    URL/rrd-graph.cgi?FLOW=<flow-name>&PERIOD=<day|week|...|1day...>&PERCENTILE=<ptile>
#
# If no FLOW is given, then graphs are generated for all rrd files 
# in the directory.
# If no PERIOD is given, then 1day is assumed.
# If no PERCENTILE is given, then 95th is assumed

source ../../conf/tmon.conf
date >> $LOGFILE

graphFlow()
{
    FLOW=$1
    PERIOD=$2

    $RRDTOOL graph --start e-$PERIOD $WORK_DIR/$FLOW-$PERIOD.png \
    -v "Bits per Second" \
    DEF:bytes_in=$CFLOWD_DB/$FLOW$RRD_FILE_SUFFIX:in_bytes:AVERAGE \
    CDEF:tin=bytes_in,$CONVERSION_RATE,* \
    AREA:tin#00CC00 \
    DEF:bytes_out=$CFLOWD_DB/$FLOW$RRD_FILE_SUFFIX:out_bytes:AVERAGE \
    CDEF:tout=bytes_out,$CONVERSION_RATE,* \
    LINE1:tout#0000FF >> $LOGFILE
}

graphUnidirectionalFlow()
{
    FLOW=$1
    PERIOD=$2

    $RRDTOOL graph --start e-$PERIOD $WORK_DIR/$FLOW-$PERIOD.png \
    -v "Bits per Second" \
    DEF:bytes_in=$CFLOWD_DB/$FLOW$RRD_FILE_SUFFIX:bytes:AVERAGE \
    CDEF:tin=bytes_in,8,* \
    AREA:tin#00CC00 >> $LOGFILE
}

htmlFlowStats()
{
    FLOW=$1
    PERIOD=$2

    $RRDTOOL fetch $CFLOWD_DB/$FLOW$RRD_FILE_SUFFIX AVERAGE -r $RESOLUTION \
    --start e-$PERIOD  \
    |  $AWK \
    -v percentile=$PERCENTILE -v conv_rate=$CONVERSION_RATE \
    -v sample_rate=$SAMPLE_RATE\
    '\
    BEGIN { \
	max_in=0; max_out=0; cum_in=0; cum_out=0; count=0; 
	Gbytes_rcvd=0; Gbytes_sent=0; 
	tstart =-1; last_t=-1; tend=-1; total_t=0; 
	getline pid < "/dev/pid"; 
	samples_file_in = sprintf("/tmp/%d_in.txt", pid);
	sort_cmd_in = sprintf("sort -rnk 2 > %s", samples_file_in);
	rm_cmd_in = sprintf("rm -f %s", samples_file_in);
	samples_file_out = sprintf("/tmp/%d_out.txt", pid);
	sort_cmd_out = sprintf("sort -rnk 3 > %s", samples_file_out);
	rm_cmd_out = sprintf("rm -f %s", samples_file_out);
    } 
    match($2,/[[:digit:]]/) { \
	count++; \
	current_t= substr($1, 0, length($1)-1); \
	if (current_t>tend) tend=current_t; \
	if (tstart < 0 || current_t < tstart) tstart=current_t; \
	current_in = $2*conv_rate; \
	current_out = $3*conv_rate; \
	if(current_in>max_in)  {max_in = current_in; } \
	if(current_out>max_out) {max_out = current_out;} \
	if(last_t>0) {\
	    delta_t=(current_t-last_t);
	    cum_in += (current_in/1e6)*delta_t; \
	    cum_out += (current_out/1e6)*delta_t; \
	    Gbytes_rcvd += delta_t*($2*sample_rate/1e9);\
	    Gbytes_sent += delta_t*($3*sample_rate/1e9);\
	    total_t += delta_t;\
	    # store samples sorted in descending order in a tmp file 
	    # for percentile calc
	    printf ("%d %f\n", delta_t, current_in) | sort_cmd_in;
	    printf ("%d %f\n", delta_t, current_out) | sort_cmd_out;
	}\
	last_t=current_t;\
    } \
    END { \
	close(sort_cmd_in); # close the sort pipe
	close(sort_cmd_out); # close the sort pipe
	# read the samples in descending order until the  desired percentile 
	# is reached
	p = (100-percentile)*total_t/100;
	t = 0;
	while ( (getline soo < samples_file_in) >0 && t<p) {
	    split(soo, sample_in);
	    t += sample_in[1];
#	    print "<br>" t/total_t " "  t " " sample_in[2];
	}
	percentile_value_in = sample_in[2];
	close(samples_file_in);
	system(rm_cmd_in);
	print "<br>"
	t = 0;
	while ( (getline boo < samples_file_out) >0 && t<p) {
	    split(boo, sample_out);
	    t += sample_out[1];
#	    print "<br>" t/total_t " "  t " " sample_out[2];
	}
	percentile_value_out = sample_out[2];
	close(samples_file_out);
	system(rm_cmd_out);

	print "<br>"
	print "<SMALL>From " strftime("%Y/%m/%d %H:%M",tstart) " to "  strftime("%Y/%m/%d %H:%M",tend) " (sampling rate "(100.0/sample_rate)"%)</SMALL>"
	if ( total_t>0 ) {
	    print "<TABLE border=0 CELLPADDING=0 CELLSPACING=0>" 
	    print "<TR>"

	    print "<TD ALIGN=right><SMALL><FONT COLOR=\"#00cc00\">In:</FONT></SMALL></TD>"
	    print "<TD WIDTH=5></TD>"

	    print "<TD ALIGN=right><SMALL>Max</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", max_in/1e6);
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"

	    print "<TD ALIGN=right><SMALL>" percentile "th ptile</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", percentile_value_in/1e6);
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"

	    print "<TD ALIGN=right><SMALL>Avg</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", (cum_in/total_t));
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"
	    print "<TD ALIGN=right><SMALL>Current</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", (current_in/1e6));
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"
	    print "<TD ALIGN=right><SMALL>Volume</SMALL></TD>" 
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Gbytes</b>", Gbytes_rcvd);
	    print "</SMALL></TD>"
	    print "</TR>"

	    print "<TR>"
	    print "<TD ALIGN=right><SMALL><FONT COLOR=\"#0000ff\">Out:</FONT></SMALL></TD>"
	    print "<TD WIDTH=5></TD>"
	    print "<TD ALIGN=right><SMALL>Max</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", max_out/1e6);
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"

	    print "<TD ALIGN=right><SMALL>" percentile "th ptile</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", percentile_value_out/1e6);
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"

	    print "<TD ALIGN=right><SMALL>Avg</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", (cum_out/total_t));
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"
	    print "<TD ALIGN=right><SMALL>Current</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Mbps</b>", (current_out/1e6));
	    print "</SMALL></TD>"
	    print "<TD WIDTH=5></TD>"
	    print "<TD ALIGN=right><SMALL>Volume</SMALL></TD>"
	    print "<TD ALIGN=right><SMALL>&nbsp;"
	    printf("<b>%.1f Gbytes</b>", Gbytes_sent);
	    print "</SMALL></TD>" 
	    print "</TR>" 
	    print "</TABLE>" 
	}
	else {
	    print "<SMALL><b>No data</b>";
	}
	print "<HR>";
    }'
}

htmlBlurbHeader()
{
    FLOW=$1

    echo "Content-type: text/html"
    echo ""
    echo "<HTML>"
    echo "<HEAD>"
    echo "<TITLE>Traffic Graph for $FLOW</TITLE>"
    echo "<STYLE TYPE=\"text/css\">"
#    echo "BODY { color : navy ; background : azure }"
    echo "H1 { color : red }"
    echo "H2 { color : blue }"
    echo "</STYLE>"
    echo "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"300\">"
    echo "<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">"
    echo "</HEAD>"
    echo "<BODY BGCOLOR=\"#ffffff\">"
    echo "<H1></H1><BR>"
#    echo "<HR>"
    echo "The statistics were last updated <B>"`date +"%A, %d %B %y at %H:%M"`"</B>"
}

htmlBlurbFlow()
{
    FLOW=$1
    PERIOD=$2

    echo "<!-- $PERIOD -->"
# the following line must be there for compat with portal traffic page   
    echo "<!-- Begin daily weekly or monthly graph-->"
##
#    echo "<HR>"
    echo "<br>Flow "$FLOW
    echo "<br>"
    echo "<IMG VSPACE=10 ALIGN=TOP " 
    echo " SRC=\"$BASE_URL/$FLOW-$PERIOD.png\" ALT=\"$PERIOD\">"
    htmlFlowStats $1 $2
# the following line must be there for compat with portal traffic page   
    echo "<!-- End daily weekly or monthly graph -->"
}

# params given on command line
FLOW=$1
PERIOD=$2

# params given via env (when run as  CGI)
if [ .$QUERY_STRING = . ]
then 
    echo "no query string" >> $LOGFILE
else
#    echo "Request from: "$REMOTE_ADDR >> $LOGFILE
#    echo "$QUERY_STRING" >> $LOGFILE
    FLOW=`echo "$QUERY_STRING" | $AWK 'BEGIN{RS="&"; FS="=";}($1 == "FLOW") { flow=$2;} END{print flow;}' -`
    PERIOD=`echo "$QUERY_STRING" | $AWK 'BEGIN{RS="&"; FS="=";}($1 == "PERIOD") { period=$2;} END{print period;}' -`
    PERCENTILE=`echo "$QUERY_STRING" | $AWK 'BEGIN{RS="&"; FS="=";}($1 == "PERCENTILE") { percentile=$2;} END{print percentile;}' -`
fi

# if PERIOD does not begin with a number append "1" in front
# so that e.g. "day" becomes "1day"
PERIOD=`echo "$PERIOD" | sed -e"s%\<[^0123456789]%1&%" -`

if [ .$PERIOD = . ]
then
    PERIOD="1day"
fi

if [ .$PERCENTILE = . ]
then 
    PERCENTILE=95
fi

if [ .$FLOW = . ]
then
# no arg given, generate day graphs for each rrd in CFLOWD_DB
    cd $CFLOWD_DB
    htmlBlurbHeader "all" $PERIOD

    for i in *$RRD_FILE_SUFFIX
    do
	echo $i >> $LOGFILE
	FLOWNAME=`echo $i | sed -e "s%$RRD_FILE_SUFFIX%%g"`
	graphFlow $FLOWNAME $PERIOD
        htmlBlurbFlow $FLOWNAME $PERIOD
    done	
else 
    graphFlow $FLOW $PERIOD
    htmlBlurbHeader $FLOW 
    htmlBlurbFlow $FLOW $PERIOD
fi



