#!/bin/sh
# Usage:  
#      rrd-stats.sh <flow_name> [<start_time> [<end_time>]]
#
# Author: nemo@invisiblehand.net
# Version: $Revision: 1.5 $, $Date: 2003/02/09 07:42:45 $
# 
# TODO: look into advantages of implementing this in perl

AWK="/bin/gawk" 
CFLOWD_DB="/usr/local/ihn/arts/data/cflowd/flows/databases"
RRDTOOL=/usr/local/ihn/RRDtool/bin/rrdtool
RESOLUTION=100
PERCENTILE=95
LOGFILE="/tmp/rrd-stats.log"
 
printUsage ()
{
    echo "Usage: "
    echo $0 "[-help (help)] [-list (list intra-period data)]  -f <flow_name> [-s <start_time> [-e <end_time>]]"
    exit 0;
}

while getopts l:f:s:e:h option
do
  case "$option"
  in
     l)     LIST="1";;
     f)     FLOW=$OPTARG;;
     s)     START_TIME=$OPTARG;;
     e)     END_TIME=$OPTARG;;
     h)     printUsage;;
  esac
done

# params given on command line

if [ .$FLOW = . ]
then
    printUsage;
fi
echo "Flow: " $FLOW 

if [ .$START_TIME = . ]
then
    START_TIME="0";
fi

if [ .$END_TIME = . ]
then
    END_TIME="now";
fi

#echo "List: "$LIST

# get step size (i.e. interval between samples) from rrd 
STEP=`$RRDTOOL info $CFLOWD_DB/$FLOW.rrd | $AWK '$1 == "step" { print $3}'`
#echo "Step: "$STEP

$RRDTOOL fetch $CFLOWD_DB/$FLOW.rrd AVERAGE -r $RESOLUTION \
    --start $START_TIME --end $END_TIME \
    |  $AWK -v list=$LIST -v percentile=$PERCENTILE '\
    BEGIN { \
	max_in=0; max_out=0; cum_in=0; cum_out=0; count=0; \
	Gbytes_rcvd=0; Gbytes_sent=0; \
	tstart =-1; last_t=-1; tend=-1; total_t=0;
	getline pid < "/dev/pid"; 
	samples_file_in = sprintf("/tmp/%d_in.txt", pid);
	sort_cmd_in = sprintf("sort -rnk 2 > %s", samples_file_in);
	rm_cmd_in = sprintf("rm -f %s", samples_file_in);
	samples_file_out = sprintf("/tmp/%d_out.txt", pid);
	sort_cmd_out = sprintf("sort -rnk 2 > %s", samples_file_out);
	rm_cmd_out = sprintf("rm -f %s", samples_file_out);
	if(list) {
	    printf("Date\t\t In\t Out\t In\t Out\n");
	    printf("\t\t Mbps\t Mbps\t Gbytes\t Gbytes\n");
	}
    } \
    match($2,/[[:digit:]]/) { \
	count++; \
	current_t= substr($1, 0, length($1)-1); \
	if (current_t>tend) tend=current_t; \
	if (tstart < 0 || current_t < tstart) tstart=current_t; \
	current_in = $2; \
	current_out = $3; \
	if(current_in>max_in)  {max_in = current_in; } \
	if(current_out>max_out) {max_out = current_out;} \
	if(last_t>0) {\
	    delta_t=(current_t-last_t);
	    cum_in += (current_in/1e6)*delta_t; \
	    cum_out += (current_out/1e6)*delta_t; \
	    Gbytes_rcvd += delta_t*(current_in/1e9);\
	    Gbytes_sent += delta_t*(current_out/1e9);\
	    total_t += delta_t;\
	    # store samples sorted in descending order in a tmp file 
	    # for percentile calc
	    printf ("%d %f\n", delta_t, current_in) | sort_cmd_in;
	    printf ("%d %f\n", delta_t, current_out) | sort_cmd_out;
	    # print out if -l option
	    if(list) {
		printf("%s\t%.2f\t%.2f\t%.2f\t%.2f\t\n", 
			strftime("%y/%m/%d %H:%M",current_t), 
			current_in*8/1e6, current_out*8/1e6,
			delta_t*(current_in/1e9),  delta_t*(current_out/1e9));
	    }
	}\
	last_t=current_t;\
    } \
    END { \
	close(sort_cmd_in); # close the sort pipe
	close(sort_cmd_out); # close the sort pipe
	# read the samples in descending order until the  desired percentile 
	# is reached
	p = (100-percentile)*total_t/100;
	t = 0;
	while ( (getline soo < samples_file_in) >0 && t<p) {
	    split(soo, sample_in);
	    t += sample_in[1];
	}
	percentile_value_in = sample_in[2];
	close(samples_file_in);
	system(rm_cmd_in);
	t = 0;
	while ( (getline boo < samples_file_out) >0 && t<p) {
	    split(boo, sample_out);
	    t += sample_out[1];
	}
	percentile_value_out = sample_out[2];
	close(samples_file_out);
	system(rm_cmd_out);
	
#	printf("\nRR samples: %d\n", count);
	printf("\nTime:\t %s\t %s\n", strftime("%Y/%m/%d %H:%M",tstart), \
		    strftime("%Y/%m/%d %H:%M",tend)); \
	printf("Avg:\t %.2f Mbps\t %.2f Mbps\n",(cum_in/total_t)*8, \
		(cum_out/total_t)*8); \
        printf("%dth p:\t %.2f Mbps\t %.2f Mbps\n", percentile, percentile_value_in*8/1e6, percentile_value_out*8/1e6);
	printf("Peak:\t %.2f Mbps\t %.2f Mbps\n", max_in*8/1000000, \
						max_out*8/1000000); \
	printf("Xfer:\t %.0f Gbytes\t %.0f Gbytes\n", Gbytes_rcvd, \
						    Gbytes_sent); \
    }'




